//
// BinaryFileIndexWriter.cpp - class to allow index generation
//

#include "StdHdr.h"

#include "BinaryFileIndexWriter.h"

#include "vos\Errors.h"
#include "vos\Time.h"

#include "BinaryFileIndex.h"

/**************************************************************************

>	BinaryFileIndexWriter::BinaryFileIndexWriter(vos::File* pIndexFile)

	Author:		Tim Browse
	Date:		2001/10/24
	Inputs:		pIndexFile - the file to write the index to.

	Purpose:	Create a new index using the specified file object.
				The index data will be saved into this file.

**************************************************************************/

BinaryFileIndexWriter::BinaryFileIndexWriter(vos::File* pIndexFile) : m_pFile(pIndexFile),
																	  m_iLastPrefix(IndexPrefix_None)
{
	COMPLAIN_IF(m_pFile == NULL,
				"NULL pointer passed to BinaryFileIndexWriter constructor!");

	// Mark index as bad until we've finished building it
	m_pFile->Write(&m_Header, sizeof(m_Header));
}

/**************************************************************************

>	BinaryFileIndexWriter::~BinaryFileIndexWriter()

	Author:		Tim Browse
	Date:		2001/10/24

	Purpose:	Destructor

**************************************************************************/

BinaryFileIndexWriter::~BinaryFileIndexWriter()
{
	m_pFile->Close();
}

/**************************************************************************

>	void BinaryFileIndexWriter::AddFile(const vos::FileSpec& FileName,
										uint32 cbFileSize,
										const vos::Time Timestamp)

	Author:		Tim Browse
	Date:		2001/10/24
	Inputs:		FileName - full pathname of the file to add.
				cbFileSize - size of the file, in bytes.
				Timestamp - modification time of the file.

	Purpose:	Add the specified file to the index.

**************************************************************************/

void BinaryFileIndexWriter::AddFile(const vos::FileSpec& FileName,
									uint32 cbFileSize,
									const vos::Time Timestamp)
{
	vos::String Leafname;
	BinaryIndexEntry Entry;
	Entry.m_Type = IndexEntryType_File;
	Entry.m_Prefix = LookupPrefix(FileName, &Leafname);
	Entry.SetFileSize(cbFileSize);
	Entry.SetTime(Timestamp);
	Entry.WriteEntry(m_pFile, Leafname.c_str());
	m_Header.m_cEntries++;
}

/**************************************************************************

>	void BinaryFileIndexWriter::AddFolder(const vos::FileSpec& FolderName,
										  const vos::Time Timestamp)

	Author:		Tim Browse
	Date:		2001/10/24
	Inputs:		FolderName - full pathname of the folder to add.
				Timestamp - modification time of the file.

	Purpose:	Add the specified folder to the index.

**************************************************************************/

void BinaryFileIndexWriter::AddFolder(const vos::FileSpec& FolderName,
									  const vos::Time Timestamp)
{
	// Make an entry for this folder
	BinaryIndexEntry Entry;
	Entry.m_Type = IndexEntryType_Folder;
	Entry.SetTime(Timestamp);
	Entry.SetFileSize(0);

	// Lookup the prefix and leafname
	vos::String Leafname;
	Entry.m_Prefix = LookupPrefix(FolderName, &Leafname);

	// Write the entry to disk
	Entry.WriteEntry(m_pFile, Leafname.c_str());

	// This folder is a possible (likely) prefix, so add it.
	AddPrefix(FolderName);

	// We've added an entry.
	m_Header.m_cEntries++;
}


/**************************************************************************

>	void BinaryFileIndexWriter::Close()

	Author:		Tim Browse
	Date:		2001/10/24

	Purpose:	Close the index file, when the index has been built.

**************************************************************************/

void BinaryFileIndexWriter::Close()
{
	// Index generation finished - mark index as good and close it.
	m_pFile->SeekFromStart(0);
	m_Header.m_Status = IndexStatus_Good;
	m_pFile->Write(&m_Header, sizeof(m_Header));
	m_pFile->Close();
}

/**************************************************************************

>	void BinaryFileIndexWriter::AddPrefix(const vos::FileSpec& FolderName)

	Author:		Tim Browse
	Date:		2001/10/24

	Purpose:	Adds the specified prefix to the list of prefixes.

**************************************************************************/

void BinaryFileIndexWriter::AddPrefix(const vos::FileSpec& FolderName)
{
	// We store the names of all folders added to the index in a 
	// 'prefix list', which allows us to find the parent index (folder)
	// of a new entry quickly.
 	vos::String Prefix = FolderName.GetString();

	// Add a trailing backslash if the prefix does not have one.
	if (Prefix[Prefix.GetLength() - 1] != '\\')
		Prefix += '\\';

	// Store the prefix index for this prefix - it's the last
	// entry we added.
	m_Prefixes[Prefix] = m_Header.m_cEntries;
}

/**************************************************************************
										
>	int BinaryFileIndexWriter::LookupPrefix(const vos::FileSpec& Name,
											vos::String* pLeafname)

	Author:		Tim Browse
	Date:		2001/10/24

	Inputs:		Name - the file/folder name whose parent (prefix) we want
					   to find.

	Outputs:	pLeafname - the leafname of the file/folder, extracted
							from 'Name'.

	Returns:	The prefix index for this file/folder name, or if the
				prefix cannot be found, IndexPrefix_None is returned.

	Purpose:	Given the full pathname of a file/folder object, find the
				prefix index to use for this fileobject, and the leafname 
				of the object.

				This is used when creating IndexEntry objects to add
				to the index.

**************************************************************************/

int BinaryFileIndexWriter::LookupPrefix(const vos::FileSpec& Name,
										vos::String* pLeafname)
{
	// Get parent folder
	vos::FileSpec fsParent;

	if (Name.GetParent(&fsParent))
	{
		// Look for this parent in our prefix list.
		vos::String strName = fsParent.GetString();

		int iPrefixFound = IndexPrefix_None;

		// Quick check for cache hit
		if ((m_iLastPrefix != IndexPrefix_None) &&
			(strName == m_LastPrefix))
		{
			// Yay - cache hit!
			iPrefixFound = m_iLastPrefix;
		}
		else
		{
			IndexPrefixes::iterator it = m_Prefixes.find(strName);

			if (it != m_Prefixes.end())
			{
				iPrefixFound = it->second;

				// Update cache
				m_LastPrefix = strName;
				m_iLastPrefix = iPrefixFound;
			}
		}

		if (iPrefixFound != IndexPrefix_None)
		{
			vos::FileSpec Leafname;
			Name.GetRelativeFileSpec(m_LastPrefix, &Leafname);
			*pLeafname = Leafname.GetString();
			return iPrefixFound;
		}
		else
		{
			::MessageBox(NULL, Name.GetString().c_str(), "Prefix not found for...", MB_OK);
		}

	}

	// Prefix not available
	*pLeafname = Name.GetString();
	return IndexPrefix_None;
}

