//
// BinaryFileIndexWriter.h - class to allow index generation
//

#ifndef inc_BinaryFileIndexWriter_h
#define inc_BinaryFileIndexWriter_h

#include "vos\File.h"

#include "FileIndexWriter.h"

#include "BinaryFileIndex.h"

#include <vector>
#include <map>

typedef std::map<vos::String, int> IndexPrefixes;

/**************************************************************************

>	class BinaryFileIndexWriter : public FileIndexWriter

	Author:		Tim Browse

	Date:		25/04/2001

	Purpose:	Create efficient binary format file indices.

				The main space saving achieved by this format is to 
				avoid storing the full pathname of each entry in the index.
				Only the leafname of each file/folder is stored in the
				index entry - the parent folder of an entry is specified
				using an index number into the array of entries.
				
				Therefore to get a full pathname for an entry, you need
				to recurse back through the index, following parent
				index links, until a null parent link is found.

				This achieves an index size approximately 30% the size
				of an equivalent text file, and can be loaded and
				searched faster.


				The file format is as follows:

					HEADER

						Version								DWORD
						 - version number of the index

						Valid								DWORD
						 - Flag to indicate state of index:
								0 = good
								1 = incomplete

						NumEntries							DWORD
						 - number of entries in this index.
						 
					ENTRIES (0..n)

						Type								DWORD
						 - Type of this entry
								0 = file
								1 = folder
								2 = end of index

						Prefix								DWORD
						 - Index of the entry that should 
						   prefix this name (or 0xFFFFFFFF
						   for no prefix - e.g. root folder
						   on a drive)

						NameLength							DWORD
						 - Length of the 'name' field, in bytes

						Name								CSTR
						 - 0-terminated leafname of this 
						   file/folder, padded to DWORD aligned
						   boundary.


**************************************************************************/

class BinaryFileIndexWriter : public FileIndexWriter
{
	public:
		// Ctor/dtor
		BinaryFileIndexWriter(vos::File* pIndexFile);
		~BinaryFileIndexWriter();

	public:
		// Add a file to the index
		virtual void AddFile(const vos::FileSpec& FileName,
							 uint32 cbFileSize,
							 const vos::Time Timestamp);

		// Add a folder to the index
		virtual void AddFolder(const vos::FileSpec& FolderName,
							   const vos::Time Timestamp);

		// Call when index has been built.
		virtual void Close();

	private:
		// Given 'Name', find the prefix folder (and leafname) to use.
		int LookupPrefix(const vos::FileSpec& Name,
						 vos::String* pLeafname);

		// Add a prefix to the prefix list.
		void AddPrefix(const vos::FileSpec& FolderName);

	private:
		// Index header - this is the first part of the file data to be saved.
		BinaryIndexHeader m_Header;

		// The file we are writing the index data to.
		vos::File* m_pFile;

		// The prefix list is used to hold the full pathnames of all 'folder'
		// objects added to the index.  This gives us a quick way of finding
		// the prefix index of a given file/folder.
		//
		// e.g. if we are trying to represent "c:\data\work\myfile.dat", we
		//		search for the prefix "c:\data\work" in the prefix list - when
		//		we find the prefix string, we then have the prefix index to
		//		use for the file.  Note we use this method for storing folder
		//		names as well as file names.
		IndexPrefixes m_Prefixes;

		// Cache - we remember the last prefix we used, as it will
		// nearly always be used for the next entry, saving us a search operation
		// for the great majority of entries.
		vos::String m_LastPrefix;
		int m_iLastPrefix;
};

#endif // !inc_BinaryFileIndexWriter_h

