//
// FastFindOptions.cpp
//

#include "StdHdr.h"

#include "FastFindOptions.h"

#include "Res\Resource.h"

#include "vgui\MessageMap.h"
#include "vgui\BrowseDialog.h"
#include "vgui\Alert.h"
#include "vgui\Question.h"

#include "vOS\Preferences.h"
#include "vOS\FileSpec.h"
#include "vOS\Folders.h"
#include "vOS\Errors.h"

#include "FastFindApp.h"
#include "FileMover.h"
#include "ProgressDialog.h"

//
// Message map for the dialog
//
// We just link the Browse button to OnBrowse().
//
BEGIN_VGUI_MESSAGE_MAP(FastFindOptions, vgui::Dialog)
	ON_VGUI_BN_CLICKED(IDC_BROWSE, OnBrowse)
	ON_VGUI_SLN_ANYCHANGE(IDC_MAXRESULTS_SLIDER, OnMaxResultsChange)
END_VGUI_MESSAGE_MAP()

/**************************************************************************

>	FastFindOptions::FastFindOptions(vgui::Window* pParent)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		pParent - parent window for this dialog.

	Purpose:	Constructor

**************************************************************************/

FastFindOptions::FastFindOptions(vgui::Window* pParent) : vgui::Dialog(IDD_OPTIONS, pParent)
{
	// TCILB
}

/**************************************************************************

>	void FastFindOptions::OnInitDialog()

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	Set up the initial dialog state - this function basically
				reads the application preferences and displays them in the
				dialog controls.

**************************************************************************/

void FastFindOptions::OnInitDialog()
{
	// Attach dialog controls
	m_ctlIndexLocation.Attach(this, IDC_INDEXLOCATION);
	m_ctlMaxResultsSlider.Attach(this, IDC_MAXRESULTS_SLIDER);
	m_ctlMaxResultsText.Attach(this, IDC_MAXRESULTS);
	m_AutoWildcards.Attach(this, IDC_AUTOWILDCARD, kOption_AutoWildcard, false);

	// Set edit box to show folder where index is kept
	m_ctlIndexLocation = FastFindApp::GetIndexLocationFileSpec().GetString();

	// Set slider range from 1...50, and display current value
	m_ctlMaxResultsSlider.SetRange(1,50);
	m_ctlMaxResultsSlider.SetPos(FastFindApp::GetMaximumResultsAllowed() / 1000);
	UpdateMaxResults();
}

/**************************************************************************

>	void FastFindOptions::OnOK()

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	Update the application preferences when the user clicks OK.

**************************************************************************/

void FastFindOptions::OnOK()
{
	// Read the new string, and see if it has changed
	vos::String NewIndexLocation = m_ctlIndexLocation;

	vos::String OldIndexLocation = FastFindApp::GetIndexLocationFileSpec().GetString();

	if (NewIndexLocation != OldIndexLocation)
	{
		vos::FileSpec fsOldIndex(FastFindApp::GetIndexFileSpec());
		vos::FileSpec fsNewIndex = NewIndexLocation;
		fsNewIndex.AppendFilename(kIndexFile);

		
		// Move the index file if it exists
		if (fsOldIndex.GetFileType() == vos::FileType_File)
		{
			// Set up the progress dialog/file mover, and set them going.
			FileMover Worker(fsOldIndex, fsNewIndex);
			vgui::StringResourceID Msg(IDS_MOVINGINDEX);
			ProgressDialog Dialog(&Worker, Msg.Load().c_str());
			Dialog.DoModal();

			TFileMoveResult Result = Worker.GetResultCode();

			// Did it work?
			if (Result != FileMover_Success)
			{
				// Report any errors with the operation.
				switch (Result)
				{
					case FileMover_UserCancelled:
					{
						// User cancelled the operation, so leave the dialog open...
						return;
					}
					
					case FileMover_NoSpace:
					{
						// Tell the user the new location could not be used.
						ReportProblem(&Worker, IDS_INDEXMOVEFAIL_NOSPACE);
						break;
					}
					
					case FileMover_FileAlreadyExists:
					{
						// Tell the user that the file already exists.
						ReportProblem(&Worker, IDS_INDEXMOVEFAILE_FILEEXISTS);
						break;
					}
					
					case FileMover_CopyFailed:
					{
						// Tell the user that the index file could not be copied
						ReportProblem(&Worker, IDS_INDEXMOVEFAIL_COPYFAILED);
						break;
					}
					
					case FileMover_DeleteFailed:
					{
						// Tell the user the old index file could not be deleted.
						ReportProblem(&Worker, IDS_INDEXMOVEFAIL_DELETEFAILED);
						break;
					}

					default:
					{
						COMPLAIN("Unknown result code in FastFindOptions::OnOK()");
						break;
					}
				}

				// There was a problem moving the index file - ask the user
				// if they want to use the new location anyway...
				vgui::StringResourceID Buttons[] = 
				{
					IDS_BUTTON_YES,
					IDS_BUTTON_NO,
					0
				};

				vgui::Question UseLocation(IDS_INDEX_USENEWLOCATION_TITLE,
										   IDS_INDEX_USENEWLOCATION);

				if (UseLocation.Ask(Buttons) == IDS_BUTTON_NO)
				{
					// User does not want to use new location, so just leave dialog open...
					return;
				}
			}
		}

		// If we get here, then the index was moved ok
		vos::FileSpec fsNewIndexLocation(NewIndexLocation);
		FastFindApp::SetIndexLocationFileSpec(fsNewIndexLocation);
	}

	// Close dialog
	Dialog::OnOK();
}

/**************************************************************************

>	void FastFindOptions::ReportProblem(FileMover* pMover,
										const vgui::StringResourceID& rMsg)

	Author:		Tim Browse
	Date:		2001/10/28
	Inputs:		pMover - the FileMover object (to get error info from)
				rMsg - the message to show to the user.

	Purpose:	Report an error, and add any error message stored by
				the file mover object to give the user more info.

**************************************************************************/

void FastFindOptions::ReportProblem(FileMover* pMover,
									const vgui::StringResourceID& rMsg)
{
	vgui::StringResourceID rTitle(IDS_INDEXMOVEFAIL_TITLE);

	vos::String strMsg = rMsg.Load();

	vos::String strErrorText;
	if (pMover->GetErrorText(&strErrorText))
	{
		// Construct a string to add to the end of the
		// dialog message that shows the reason for the error.
		vgui::StringResourceID rReason(IDS_ERROR_REASON);
		vos::String strReason;
		rReason.Printf(&strReason, strErrorText.c_str());

		strMsg += "\n\n";
		strMsg += strReason;
	}

	vgui::Alert::Show(rTitle.Load(), strMsg, this);
}

/**************************************************************************

>	bool FastFindOptions::OnBrowse()

	Author:		Tim Browse
	Date:		2001/10/25
	Returns:	true => message always handled.

	Purpose:	Allow the user to use the system folder browse dialog to
				specify a new place to keep the index file.

**************************************************************************/

bool FastFindOptions::OnBrowse()
{
	vgui::BrowseDialog Browser((vgui::BrowseDialogFlags) (vgui::bdf_ObjectMustExist|vgui::bdf_ShowFolderName), 
							   this);

	vos::FileSpec fsNewLocation;

	if (Browser.BrowseForFolder("Select index location...", "", &fsNewLocation))
	{
		// Set edit control to contain result
		m_ctlIndexLocation = fsNewLocation.GetString();
	}

	return true;
}

bool FastFindOptions::OnMaxResultsChange()
{
	int NewPos = m_ctlMaxResultsSlider.GetPos();

	FastFindApp::SetMaximumResultsAllowed(NewPos * 1000);
	UpdateMaxResults();
	return true;
}

void FastFindOptions::UpdateMaxResults()
{
	vgui::StringResourceID rFormat(IDS_THOUSANDS);

	vos::String strMaxResults;
	rFormat.Printf(&strMaxResults, 
				   FastFindApp::GetMaximumResultsAllowed() / 1000);

	m_ctlMaxResultsText.SetText(strMaxResults);
}