//
// FileIndex.cpp
//

#include "StdHdr.h"

#include "FileIndex.h"

#include "vgui\ProgressDialog.h"

#include "vos\DiskFile.h"

#include <deque>

#include "IndexSearcher.h"

#include "Res\Resource.h"

/**************************************************************************

>	FileIndex::FileIndex()

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	Constructor

**************************************************************************/

FileIndex::FileIndex() : m_pIndexData(NULL),
						 m_pEntries(NULL),
						 m_pSearchResults(NULL),
						 m_cSearchResults(0),
						 m_pHeader(NULL),
						 m_cFiles(0), 
						 m_cFolders(0),
						 m_cbTotalFileSize(0)

{
	// TCILB
}

/**************************************************************************

>	FileIndex::~FileIndex()

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	Destructor

**************************************************************************/

FileIndex::~FileIndex()
{
	CleanUp();
}



/**************************************************************************

>	vos::Error FileIndex::Load(const vos::FileSpec fsIndex)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		fsIndex - filename of the index file to load.
	Returns:	Error

	Purpose:	Load the index data from the specified index file into
				this index object.

**************************************************************************/

vos::Error FileIndex::Load(const vos::FileSpec fsIndex)
{
	//
	// Discussion:
	//
	// Loading an index is very simple - we find out how big the file is,
	// and attempt to load it into RAM in one step.
	//
	// The index contains the leafnames of every file and folder on
	// the volumes that were indexed (and the parent folder of each entry).
	//
	// We then build a secondary index (in m_pEntries, using SetUpIndices())
	// which is simply an array of pointers that point into the block of
	// index data we loaded.  We do this because the entries in the raw index
	// data are of variable length, so the secondary index gives us a constant
	// time lookup into the main index.
	//
	// We use variable length records because filenames can be hundreds of 
	// characters long, but are usually smaller than that - less than 15 characters
	// is common.  Therefore we avoid wasting space by storing variable length 
	// records.
	//

	// Check the file exists
	if (fsIndex.GetFileType() != vos::FileType_File)
	{
		// Not a file, or it does not exist, so complain about it
		COMPLAIN("Index file not found!");
		return ERROR(Err_GenericError, 
					 "The index file was not found.");
	}

	// Find out how big the file is
	m_cIndexDataSize = fsIndex.GetFileSize();

	if (m_cIndexDataSize > 0)
	{
		// Allocate the index data buffer
		m_pIndexData = new byte[m_cIndexDataSize];

		// Load the data into the buffer
		vos::DiskFile IndexFile;
		if (IndexFile.Open(fsIndex, vos::fam_Read))
		{
			if (IndexFile.Read(m_pIndexData, m_cIndexDataSize))
			{
				// Data read ok
				m_pHeader = (BinaryIndexHeader*) m_pIndexData;
				IndexFile.Close();
			}
			else
			{
				// Error reading data
				IndexFile.Close();
				CleanUp();
				return ERROR(Err_GenericError, 
							 "The index file data could not be loaded.");
			}
		}
		else
		{
			CleanUp();
			return ERROR(Err_GenericError, 
						 "The index file could not be opened to load the data.");
		}
	}
	else
	{
		return ERROR(Err_GenericError,
					 "The index file does not contain any data.");
	}

	if (m_pHeader->m_Version > kBinaryIndexVersion)
	{
		return ERROR(Err_GenericError,
					 "Index file was built by a newer version of FastFind "
					 "(and cannot be used)");
	}

	// Set up the secondary indices
	if (!SetUpIndices((BinaryIndexEntry*) (m_pHeader + 1), m_cIndexDataSize))
	{
		return LASTERROR;
	}

	// Data loaded ok
	return VNOERROR;
}

/**************************************************************************

>	void FileIndex::CleanUp()

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	Clean up all the data structures we use to maintain the
				index.

**************************************************************************/

void FileIndex::CleanUp()
{
	// Unload the index
	delete m_pIndexData;
	m_pIndexData = NULL;
	m_cIndexDataSize = 0;
	m_pHeader = NULL;

	// Clean up the index entry array
	delete m_pEntries;
	m_pEntries = NULL;

	// Clean up search results
	delete m_pSearchResults;
	m_pSearchResults = NULL;
	m_cSearchResults = 0;
}

/**************************************************************************

>	vos::Error FileIndex::SetUpIndices(BinaryIndexEntry* pIndexData, int cbIndexSize)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		pIndexData - the entries in this index
				cbIndexSize - the size of the index in bytes.  The function
							  should not try to access more than this
							  amount of data in the block pointed to by
							  pIndexData.
	Returns:	Error if index is not decoded correctly
	
	Purpose:	Set up the secondary index in m_pEntries to point into
				the main index data block, which gives us constant time
				lookup for a particular entry in the index (given that
				we have its index).

				We also set up a null search results array in 
				m_pSearchResults.

**************************************************************************/

vos::Error FileIndex::SetUpIndices(BinaryIndexEntry* pIndexData, int cbIndexSize)
{
	m_cFiles = 0;
	m_cFolders = 0;
	m_cbTotalFileSize = 0;

	// You shouldn't be calling this if no index data has been loaded.
	if (m_pIndexData == NULL)
	{
		COMPLAIN("No index data in SetUpIndices()!");
		return ERROR(Err_LogicError, 
					 "FastFind has encountered an internal fault - please report this problem.");
	}

	// Calculate end of the index data
	
	// Allocate the secondary index buffer
	m_pEntries = new BinaryIndexEntry*[m_pHeader->m_cEntries];

	// Set up the pointers in the index
	BinaryIndexEntry* pEntry = pIndexData;

	// Only use as much data as we are supposed to
	int cbDataLeft = cbIndexSize;

	for (int iEntry = 0; iEntry < m_pHeader->m_cEntries; iEntry++)
	{
		// Set up pointer to the index
		m_pEntries[iEntry] = pEntry;

		// Update running totals
		switch (pEntry->m_Type)
		{
			case IndexEntryType_File:
			{
				m_cFiles++;
				m_cbTotalFileSize += pEntry->GetFileSize();
				break;
			}

			case IndexEntryType_Folder:
			{
				m_cFolders++;	
				break;
			}

			default:
			{
				COMPLAIN("Unknown index object type!");
				break;
			}
		}

		// Move on to the next entry
		int SizeOfEntry = sizeof(BinaryIndexEntry) + pEntry->m_NameLength;
		pEntry = (BinaryIndexEntry*) (((byte*) pEntry) + SizeOfEntry);

		// Check we have enough data left
		cbDataLeft -= SizeOfEntry;
		if (cbDataLeft < 0)
		{
			// Error - we have run off the end of this index, which probably
			// means it is in a format we don't understand
			delete [] m_pEntries;
			m_pEntries = NULL;
			return ERROR(Err_GenericError, 
						 "Warning: Index file is not understood by this version of FastFind.");
		}
	}

	// Initialise the search results array
	m_pSearchResults = new BinaryIndexEntry*[m_pHeader->m_cEntries];
	m_cSearchResults = 0;

	// All done
	return VNOERROR;
}

/**************************************************************************

>	bool FileIndex::Search(const vos::String& strPattern,
						   bool bUseRegExp,
						   bool bAutoWildcard,
						   vgui::Window* pParentWindow)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		strPattern - the filename/pattern to look for.
				bUseRegExp - if true, then strPattern is a proper regular
							 expression - if false, then it is a standard
							 wildcarded filespec (e.g "*.doc" etc)
				bAutoWildcard - if true, then if the pattern is not a regular 
								expression, it is prepended and appended with
								"*", to allow simple text searches to match
								partial filenames.
				pParentWindow - the parent window for the search op
								(for UI modal dialogs etc)

	Returns:	true => search completed ok
				false => strPattern contains syntax error (for RegExp)

	Purpose:	Search the index for a specified filename/pattern.
		
				Use the following functions to analyse the results:

					- GetNumberOfSearchResults()
					- GetSearchResult(int iSearchResult)
					- GetFullPathname(int iSearchResult)

				NB. File pattern (wildcard) search not yet supported.

**************************************************************************/

bool FileIndex::Search(const vos::String& strPattern, 
					   bool bUseRegExp,
					   bool bAutoWildcard,
					   vgui::Window* pParentWindow)
{
	IndexSearcher Searcher;

	if (!Searcher.Init(this, strPattern, bUseRegExp, bAutoWildcard))
	{
		// Failed to init - probably a regexp error
		return false;
	}

	vgui::StringResourceID rSearchTitle(IDS_SEARCHPROGRESS_TITLE);

	vgui::ProgressDialog SearchProgress(&Searcher, rSearchTitle.Load(), pParentWindow);
	SearchProgress.ShowProgressWithoutCancel();

	// ok, search completed ok
	return true;
}

void FileIndex::ClearSearchResults()
{
	m_cSearchResults = 0;
}

void FileIndex::AddSearchResult(BinaryIndexEntry* pResult)
{
	m_pSearchResults[m_cSearchResults] = pResult;
	m_cSearchResults++;
}

/**************************************************************************

>	bool FileIndex::FindSubstring(const char *szString, const char* szSubString)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		szString - the string to search
				szSubString - the sub-string to search for
	Returns:	true => substring found;
				false otherwise.

	Purpose:	Find out if one substring is present in another string.

**************************************************************************/

bool FileIndex::FindSubstring(const char *szString, const char* szSubString)
{
	int StringLength = strlen(szString);
	int SubStringLength = strlen(szSubString);

	// This is how much of the first string we should search (because after
	// that the substring cannot fit into the remainder of the string).
	int SearchLength = StringLength - SubStringLength;

	for (int i = 0; i <= SearchLength; i++)
	{
		// Is this a possible starting match?
		if (tolower(szString[i]) == tolower(szSubString[0]))
		{
			// Yes, so check it
			if (_strnicmp(szString+i, szSubString, SubStringLength) == 0)
				return true;
		}
	}

	// Not found
	return false;
}

/**************************************************************************

>	int FileIndex::GetNumberOfSearchResults() const

	Author:		Tim Browse
	Date:		2001/10/25
	Returns:	Number of files found.

	Purpose:	Find out how many files were found by the last call
				to Search().

**************************************************************************/

int FileIndex::GetNumberOfSearchResults() const
{
	return m_cSearchResults;
}

/**************************************************************************

>	BinaryIndexEntry* FileIndex::GetSearchResult(int iSearchResult) const

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		iSearchResult - index of the result to be accessed.
	Returns:	The index entry corresponding to this search result.

	Purpose:	Find the specified search result from the last call to
				Search().

**************************************************************************/

BinaryIndexEntry* FileIndex::GetSearchResult(int iSearchResult) const
{
	if (iSearchResult < m_cSearchResults)
	{
		return m_pSearchResults[iSearchResult];
	}

	COMPLAIN("GetSearchResult() - index out of bounds!");
	return NULL;
}


/**************************************************************************

>	bool FileIndex::SearchResultIsFolder(int iSearchResult) const

	Author:		Tim Browse
	Date:		14/2/2002
	Inputs:		iSearchResult - index of the result to be accessed.
	Returns:	true => search result is a folder
				false => search result is a file

	Purpose:	Find out if the specified search result describes a folder.

**************************************************************************/

bool FileIndex::SearchResultIsFolder(int iSearchResult) const
{
	if (iSearchResult < m_cSearchResults)
	{
		return (m_pSearchResults[iSearchResult]->m_Type == IndexEntryType_Folder);
	}

	COMPLAIN("SearchResultIsFolder() - index out of bounds!");
	return false;
}

/**************************************************************************

>	uint32 FileIndex::GetFileSize(int iSearchResult) const

	Author:		Tim Browse
	Date:		2001/11/01
	Returns:	size, in bytes, of the file.

	Purpose:	Get the file size of the specified search result.

**************************************************************************/

uint32 FileIndex::GetFileSize(int iSearchResult) const
{
	if (iSearchResult < m_cSearchResults)
	{
		return m_pSearchResults[iSearchResult]->GetFileSize();
	}

	COMPLAIN("GetFileSize() - index out of bounds!");
	return NULL;
}

/**************************************************************************

>	vos::Time FileIndex::GetTimeStamp(int iSearchResult) const

	Author:		Tim Browse
	Date:		8/4/2002
	Inputs:		iSearchResult - the search result to query
	Returns:	Timestamp of the specified file.

	Purpose:	Get the timestamp of the file/folder.

**************************************************************************/

vos::Time FileIndex::GetTimeStamp(int iSearchResult) const
{
	if (iSearchResult < m_cSearchResults)
	{
		return m_pSearchResults[iSearchResult]->GetTime();
	}

	COMPLAIN("GetTimeStamp() - index out of bounds!");
	return vos::Time();
}

/**************************************************************************

>	vos::String FileIndex::GetFullPathname(int iSearchResult) const

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		iSearchResult - index of the result to be accessed.
	Returns:	The full pathname of the specified search result.

	Purpose:	Get the full pathname of the specified search result.

**************************************************************************/

vos::String FileIndex::GetFullPathname(int iSearchResult) const
{
	BinaryIndexEntry *pEntry = m_pSearchResults[iSearchResult];
	return GetFullPathname(pEntry);
}

/**************************************************************************

>	vos::String FileIndex::GetLeafname(int iSearchResult) const

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		iSearchResult - index of the result to be accessed.
	Returns:	The leafname of the specified search result.

	Purpose:	Get the leafname of the specified search result.

**************************************************************************/

vos::String FileIndex::GetLeafname(int iSearchResult) const
{
	BinaryIndexEntry *pEntry = m_pSearchResults[iSearchResult];
	return vos::String(pEntry->GetName());
}
/**************************************************************************

>	vos::String FileIndex::GetFolder(int iSearchResult) const

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		iSearchResult - index of the result to be accessed.
	Returns:	The name of the folder containing the specified search result.

	Purpose:	Get the name of the folder containing the specified search 
				result.

**************************************************************************/

vos::String FileIndex::GetFolder(int iSearchResult) const
{
	BinaryIndexEntry *pEntry = m_pSearchResults[iSearchResult];
	vos::FileSpec fsFolder;
	GetFolder(pEntry, &fsFolder);
	return fsFolder.GetString();
}

/**************************************************************************

>	void FileIndex::GetFolder(BinaryIndexEntry *pEntryToName,
							  vos::FileSpec* pfsFolder) const

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		pEntryToName - the index entry to build the folder name for.
	Outputs:	pfsFolder - the folder name
	Returns:	The folder that contains the specified entry.

	Purpose:	Given an index entry, extract the full pathname of the
				folder that contains it, by recursing through the parent 
				links in the index.

**************************************************************************/

void FileIndex::GetFolder(BinaryIndexEntry *pEntryToName,
						  vos::FileSpec* pfsFolder) const
{
	// Build up folder prefix list
	std::deque<int> m_Names;

	BinaryIndexEntry *pEntry = pEntryToName;

	// We use a deque, and push the prefix indices onto the front, so
	// we get the correct ordering of parent links...
	while (pEntry->m_Prefix != IndexPrefix_None)
	{
		m_Names.push_front(pEntry->m_Prefix);
		pEntry = m_pEntries[pEntry->m_Prefix];

		// Sanity check - prefixes should always be folders.
		COMPLAIN_IF(pEntry->m_Type != IndexEntryType_Folder,
					"Invalid prefix found - not a folder!");
	}

	// ...and now construct parent folder name for item.
	pfsFolder->Clear();

	std::deque<int>::iterator it = m_Names.begin();

	while (it != m_Names.end())
	{
		pfsFolder->AppendFolder(m_pEntries[*it]->GetName());
		++it;
	}
}

/**************************************************************************

>	vos::String FileIndex::GetFullPathname(BinaryIndexEntry *pEntryToName) const

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		pEntryToName - the index entry to build the pathname for.
	Returns:	The full pathname of the specified entry.

	Purpose:	Given an index entry, extract the full pathname of the
				entry by recursing through the parent links in the index.

**************************************************************************/

vos::String FileIndex::GetFullPathname(BinaryIndexEntry *pEntryToName) const
{
	// Get parent folder
	vos::FileSpec Pathname;
	GetFolder(pEntryToName, &Pathname);

	// Add the leafname
	Pathname.AppendFilename(pEntryToName->GetName());

	// Return result to caller.
	return Pathname.GetString();
}

/**************************************************************************

>	uint32 FileIndex::GetSearchTotalFileSize() const

	Author:		Tim Browse
	Date:		30/5/2002
	Returns:	Size, in bytes, of all matching files.

	Purpose:	Find out the total size of all matching files in the
				index (i.e. those in the current search)

**************************************************************************/

uint32 FileIndex::GetSearchTotalFileSize() const
{
	int cResults = GetNumberOfSearchResults();

	uint32 cbTotalFileSize = 0;

	for (int iResult = 0; iResult < cResults; iResult++)
	{
		cbTotalFileSize += m_pSearchResults[iResult]->GetFileSize();
	}

	return cbTotalFileSize;
}


/**************************************************************************

>	void FileIndex::WriteTextIndex(const vos::FileSpec& FS)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		FS - the file to create with the text version of the index.

	Purpose:	Test function - produces a text format index from a 
				binary index file.  Used to check the correctness of the
				binary index generation routines.

**************************************************************************/

void FileIndex::WriteTextIndex(const vos::FileSpec& FS)
{
	vos::DiskFile DiskFile;

	if (DiskFile.Open(FS, vos::fam_Write))
	{
		for (int iEntry = 0; iEntry < m_pHeader->m_cEntries; iEntry++)
		{
			// Output this entry as text
			vos::String Pathname = GetFullPathname(m_pEntries[iEntry]);

			// Add trailing backslash for folders
			if (m_pEntries[iEntry]->m_Type == IndexEntryType_Folder)
			{
				if (Pathname.Right(1) != "\\")
					Pathname += "\\";
			}

			// Write to file
			if (!DiskFile.WriteLine(Pathname.c_str()))
			{
				COMPLAIN("WriteTextIndex() Unable to write to the index file!");
				ERROR_HANDLED();
				return;
			}
		}

		DiskFile.Close();
	}
	else
	{
		COMPLAIN("Unable to open text index file!");
		ERROR_HANDLED();
	}
}
