//
// ProgressWorker.h
//

#ifndef inc_ProgressWorker_h
#define inc_ProgressWorker_h

#include "vgui\vgui.h"

//
// TProgressWorkerStatus
//
// Used by the ProgressWorker_Timesliced interface class to signify
// whether or not the work has been completed.
//
typedef enum
{
	ProgressWorker_Finished,
	ProgressWorker_NotFinished
}
TProgressWorkerStatus;

class ProgressDialog;

/**************************************************************************

>	class ProgressWorker_Timesliced

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	A callback interface to use to carry out work while a
				progress dialog is open.  The worker must carry out the
				work a piece at a time (in a timesliced manner).

				The ProgressDialog will call DoWork() periodically
				(driven by idle events) until the handler signifies that
				the job is complete (by returning ProgressWorker_Finished).
				The amount of time the worker spends before returning
				directly affects the responsiveness of the UI - the worker
				should try to return within <200ms to keep the UI
				working smoothly.

				If the user presses Cancel, the dialog calls OnCancel()
				to inform the worker object, which is expected to have
				performed all necessary cleanup by the time it returns.

**************************************************************************/

class ProgressWorker_Timesliced
{
	public:
		// Prepare the worker object, ready to start the work.
		// Called once before DoWork() or OnCancel() are called.
		virtual void Prepare() = 0;

		// Do a block of work, and let the ProgressDialog know how much of
		// the work has been completed so far, and whether the work has
		// been finished or not.
		virtual TProgressWorkerStatus DoWork(int* pPercentComplete) = 0;

		// The user has clicked the "Cancel" button, so the worker object
		// should perform any shutdown necessary before returning.  The
		// object will not be called by the ProgressDialog at all after
		// this function has been called.
		virtual void OnCancel(vgui::Dialog* pProgressDialog) = 0;
};

/**************************************************************************

>	class ProgressWorker_OneShot

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	A callback interface to use to carry out work while a
				progress dialog is open.  The worker object should do the
				whole job at once, updating the progress display periodically
				(see function descriptions below).

**************************************************************************/

class ProgressWorker_OneShot
{
	public:
		// Prepare the worker object, ready to start the work.
		// Called once before DoWork().
		virtual void Prepare() = 0;

		// Do the entire job.  The worker object should call the function
		// ProgressDialog::UpdateProgressDisplay() regularly to keep the
		// progress display accurate, and the UI responsive.  The worker
		// should try to call this function every 200ms to keep the UI
		// running smoothly.
		// The worker must also call ProgressDisplay::Cancelled() to find out
		// if the user has asked for the job to be cancelled.  If this returns
		// true, then the worker must do any necessary clean-up, and return
		// from this function.
		virtual void DoWork(ProgressDialog* pProgressDialog) = 0;
};

#endif // !inc_ProgressWorker_h
