//
// TextFileIndexWriter.cpp - class to allow index generation
//

#include "StdHdr.h"

#include "TextFileIndexWriter.h"

#include "vos\Errors.h"
#include "vos\Time.h"


/**************************************************************************

>	TextFileIndexWriter::TextFileIndexWriter(vos::File* pIndexFile)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		pIndexFile - the file to write the index data to.

	Purpose:	Constructor

**************************************************************************/

TextFileIndexWriter::TextFileIndexWriter(vos::File* pIndexFile) : m_pFile(pIndexFile)
{
	COMPLAIN_IF(m_pFile == NULL,
				"NULL pointer passed to TextFileIndexWriter constructor!");

	// Mark index as bad until we've finished building it
	m_pFile->SeekFromStart(0);
	m_pFile->WriteLine(kIndexMarker_Bad);
}

/**************************************************************************

>	TextFileIndexWriter::~TextFileIndexWriter()

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	Destructor

**************************************************************************/

TextFileIndexWriter::~TextFileIndexWriter()
{
	m_pFile->Close();
}

/**************************************************************************

>	void TextFileIndexWriter::AddFile(const vos::FileSpec& FileName,
									  uint32 cbFileSize,
									  const vos::Time Timestamp)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		FileName - full pathname of the file to add.
				cbFileSize - size of the file, in bytes.
				Timestamp - modification time of the file.

	Purpose:	Add a file object to the index.

**************************************************************************/

void TextFileIndexWriter::AddFile(const vos::FileSpec& FileName,
								  uint32 cbFileSize,
								  const vos::Time Timestamp)
{
	m_pFile->WriteLine(FileName.GetString());
}

/**************************************************************************

>	void TextFileIndexWriter::AddFolder(const vos::FileSpec& FolderName,
										const vos::Time Timestamp)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		FolderName - full pathname of the folder to add to the index.
				Timestamp - modification time of the file.

	Purpose:	Add a folder object to the index.

**************************************************************************/

void TextFileIndexWriter::AddFolder(const vos::FileSpec& FolderName,
									const vos::Time Timestamp)
{
	// Write name with trailing slash appended to mark it as a folder
	vos::String strFolderName = FolderName.GetString();
	m_pFile->Write(strFolderName.c_str(), strFolderName.GetLength(), NULL);

	// Add a terminating backslash for folders (if it's not a root folder)
	if (strFolderName.Right(1) != "\\")
	{
		m_pFile->WriteLine("\\");
	}
	else
	{
		m_pFile->WriteLineEnd();
	}
}


/**************************************************************************

>	void TextFileIndexWriter::Close()

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	Close down the index file, and mark it as 'good', because
				we've finished writing it.

**************************************************************************/

void TextFileIndexWriter::Close()
{
	// Index generation finished - mark index as good and close it.
	m_pFile->SeekFromStart(0);
	m_pFile->WriteLine(kIndexMarker_Good);
	m_pFile->Close();
}
