//
// VolumeIndexBuilder.cpp
//

#include "StdHdr.h"

#include "VolumeIndexBuilder.h"

#include "FileIndexBuilder.h"
#include "FileIndexWriter.h"

/**************************************************************************

>	VolumeIndexBuilder::VolumeIndexBuilder(IndexBuilderMonitor* pMonitor)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		pMonitor - the object that will monitor the index building 
						   process.

	Purpose:	Constructor

				The constructor builds a list of logical fixed drives
				on the system, ready to index them.

**************************************************************************/

VolumeIndexBuilder::VolumeIndexBuilder(IndexBuilderMonitor* pMonitor) : m_pBgIndexFile(NULL),
																		m_pIndexBuilder(NULL),
																		m_bBgIndexFinished(false),
																		m_pBuildMonitor(pMonitor)
{
#ifdef VGUI_MSWINDOWS
	//
	// We need to get down and dirty with Win32 here - until I abstract this
	// into vOS anyway.
	//
	// GetLogicalDriveStrings() returns a sequence of null-terminated strings
	// that represent the root filenames of each of the logical drives
	// found on the system...
	//

	// Find out the size of buffer needed for drive strings
	DWORD dwBufLen = ::GetLogicalDriveStrings(0, NULL);

	// Ok, allocate the buffer required
	m_pDrives = new char[dwBufLen+1];

	// And get the drive strings
	// TODO: Report error if any occurs
	dwBufLen = ::GetLogicalDriveStrings(dwBufLen, m_pDrives);

	// Set the current drive to point at the first drive
	m_pCurrentDrive = m_pDrives;

	// Find out what kind of drive the current drive is - fixed, removable,
	// CDROM, or RAM disk, and so on.
	m_DriveType = ::GetDriveType(m_pCurrentDrive);
#else
#error This function needs porting!
#endif
}

/**************************************************************************

>	VolumeIndexBuilder::~VolumeIndexBuilder()

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	Destructor

**************************************************************************/

VolumeIndexBuilder::~VolumeIndexBuilder()
{
	delete m_pIndexBuilder;
	delete [] m_pDrives;
}

/**************************************************************************

>	vos::Error VolumeIndexBuilder::BuildIndexes(FileIndexWriter* pIndexFile)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		pIndexFile - the index generator to use to write the index.

	Purpose:	Build the index for all drives in one operation - this
				function will not return until the index generation is
				complete, which may be of the order of minutes.

**************************************************************************/

vos::Error VolumeIndexBuilder::BuildIndexes(FileIndexWriter* pIndexFile)
{
	return BuildIndexes(pIndexFile, INT_MAX, NULL);
}

/**************************************************************************
											
>	vos::Error VolumeIndexBuilder::BuildIndexes(FileIndexWriter* pIndexFile,
												int cFilesToIndex, 
												bool *pbFinished)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		pIndexFile - the index generator to use to write the index.
				cFilesToIndex - the maximum number of files/folders that
								should be added to the index in this pass.
	Outputs:	pbFinished - true => index building complete
							 false => there are still files left to index.
	Returns:	Error

	Purpose:	Build the index for all volumes on the system, in an 
				interruptible way - the caller provides the number of
				files that should be indexed before the function
				returns.

				This allows the index to be built while still providing a
				responsive UI that is updated often enough, without
				having to resort to multi-threading.

**************************************************************************/

vos::Error VolumeIndexBuilder::BuildIndexes(FileIndexWriter* pIndexFile,
											int cFilesToIndex, 
											bool *pbFinished)
{
	while (strlen(m_pCurrentDrive) > 0)
	{
		// Build index for this drive if it is local
		if (m_DriveType == DRIVE_FIXED)
		{
			if (m_pIndexBuilder == NULL)
			{
				// Create an index builder for this drive
				m_pIndexBuilder = new FileIndexBuilder(pIndexFile, 
													   vos::FileSpec(m_pCurrentDrive), 
													   NULL);
			}

			// Index as many as we can
			bool bFinished;
			vos::Error Result = m_pIndexBuilder->BuildIndex(cFilesToIndex, &bFinished);

			// If not finished, then we've hit the limit so return.
			if (!bFinished)
			{
				if (pbFinished != NULL)
					*pbFinished = false;

				// Tell monitor what's going on
				if (m_pBuildMonitor != NULL)
				{
					vos::FileSpec CurrentFolder;
					m_pIndexBuilder->GetCurrentFolder(&CurrentFolder);
					m_pBuildMonitor->UpdateStatus(CurrentFolder);
				}

				return Result;
			}

			// Propogate any errors
			if (!Result)
				return LASTERROR;

			// We have finished this drive, so delete the indexer for the
			// drive, and fall through and try the next drive.
			delete m_pIndexBuilder;
			m_pIndexBuilder = NULL;
		}

		// Try next drive
		m_pCurrentDrive += strlen(m_pCurrentDrive) + 1;

		if (strlen(m_pCurrentDrive) > 0)
			m_DriveType = ::GetDriveType(m_pCurrentDrive);
	}

	// Run out of drives, so we've finished.
	if (pbFinished != NULL)
	{
		*pbFinished = true;
	}

	return VNOERROR;
}



/**************************************************************************

>	vos::Error VolumeIndexBuilder::StartIndexing(FileIndexWriter* pIndexFile)

	Author:		Tim Browse
	Date:		22/04/2001
	Inputs:		pIndexFile - the file to put the index data into.
	Returns:	Any errors encountered during startup of background	
				indexing.

	Purpose:	Start background indexing.  The object uses the vGUI idle
				handler mechanism to schedule the work.

**************************************************************************/

vos::Error VolumeIndexBuilder::StartIndexing(FileIndexWriter* pIndexFile)
{
	// Ensure we're not already doing background indexing.
	if (m_pBgIndexFile != NULL)
	{
		COMPLAIN("StartIndexing() called when already indexing!");
		return ERROR(Err_LogicError, 
					 "The volume indexing system was asked to start "
					 "indexing when an indexing operation was already "
					 "in progress.");
	}

	// Ok, start off the indexing
	m_pBgIndexFile = pIndexFile;
	m_bBgIndexFinished = false;
	ClaimIdleEvents();
	return VNOERROR;
}


/**************************************************************************

>	void VolumeIndexBuilder::StopIndexing()

	Author:		Tim Browse
	Date:		22/04/2001

	Purpose:	Stop background indexing - this stops all indexing
				activity - if indexing is started again using StartIndexing()
				then the indexing starts from scratch.

**************************************************************************/

void VolumeIndexBuilder::StopIndexing()
{
	// Disable indexing - the indexing will shutdown on the next idle event.
	m_pBgIndexFile = NULL;
}


/**************************************************************************

>	void VolumeIndexBuilder::PauseIndexing()

	Author:		Tim Browse
	Date:		22/04/2001

	Purpose:	Temporarily halt background indexing until ContinueIndexing()
				is called (or StopIndexing() is called to stop it altogether).

				UNIMPLEMENTED!

**************************************************************************/

void VolumeIndexBuilder::PauseIndexing()
{
	COMPLAIN("UNIMPLEMENTED!");
}


/**************************************************************************

>	void VolumeIndexBuilder::ContinueIndexing()

	Author:		Tim Browse
	Date:		22/04/2001

	Purpose:	Continue background indexing after it has been paused - see
				PauseIndexing().

				UNIMPLEMENTED!

**************************************************************************/

void VolumeIndexBuilder::ContinueIndexing()
{
	COMPLAIN("UNIMPLEMENTED!");
}



/**************************************************************************

>	bool VolumeIndexBuilder::IsIndexing(vos::FileSpec *pCurrentFolder = NULL)

	Author:		Tim Browse
	Date:		22/04/2001
	Outputs:	pCurrentFolder - the folder currently being indexed
								 (may be NULL if not needed)
								 (may be empty if indexing not yet started)
	Returns:	true => currently doing background indexing
				false=> not currently indexing

	Purpose:	Find out if the object is currently doing background
				indexing (and, if it is, what folder it has got up to)

**************************************************************************/

bool VolumeIndexBuilder::IsIndexing(vos::FileSpec *pCurrentFolder)
{
	// If we have an index file, we're indexing.
	if (m_pBgIndexFile != NULL)
	{
		// Fill in current folder if they want it...
		if (pCurrentFolder != NULL)
		{
			if (m_pIndexBuilder != NULL)
			{
				m_pIndexBuilder->GetCurrentFolder(pCurrentFolder);
			}
		}

		// ...and, yes, we are indexing.
		return true;
	}
	else
	{
		// Not indexing
		return false;
	}
}


/**************************************************************************

>	bool VolumeIndexBuilder::OnIdle()

	Author:		Tim Browse
	Date:		22/04/2001
	Returns:	true  => continue receiving idle events.
				false => we're done

	Purpose:	Idle event handler - does the background event indexing.

**************************************************************************/

bool VolumeIndexBuilder::OnIdle(uint iIdleEvent)
{
	// Check to see if bg indexing has been stopped
	if (m_pBgIndexFile == NULL)
	{
		// Yes, no index file => indexing stopped, so we're done.
		ReleaseIdleEvents();
		return false;
	}

	// Do the next chunk of indexing
	if (BuildIndexes(m_pBgIndexFile, 500, &m_bBgIndexFinished))
	{
		// Index building successful - if we need to keep going, tell
		// the app framework.
		if (m_bBgIndexFinished)
		{
			// All done, so clean up and deregister idle handler
			m_pBgIndexFile->Close();
			m_pBgIndexFile = NULL;

			// Tell monitor what's going on
			if (m_pBuildMonitor != NULL)
			{
				m_pBuildMonitor->IndexComplete();
			}

			ReleaseIdleEvents();
			return false;
		}
		else
		{
			// Not done yet, so request more idle events
			return true;
		}
	}
	else
	{
		// An error occured - complain for now
		COMPLAIN("An error occured while indexing.");
		ERROR_HANDLED();

		// We're done indexing.
		m_pBgIndexFile = NULL;
		ReleaseIdleEvents();
		return false;
	}
}

